/**
* OLAT - Online Learning and Training<br>
* http://www.olat.org
* <p>
* Licensed under the Apache License, Version 2.0 (the "License"); <br>
* you may not use this file except in compliance with the License.<br>
* You may obtain a copy of the License at
* <p>
* http://www.apache.org/licenses/LICENSE-2.0
* <p>
* Unless required by applicable law or agreed to in writing,<br>
* software distributed under the License is distributed on an "AS IS" BASIS, <br>
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
* See the License for the specific language governing permissions and <br>
* limitations under the License.
* <p>
* Copyright (c) since 2004 at Multimedia- & E-Learning Services (MELS),<br>
* University of Zurich, Switzerland.
* <hr>
* <a href="http://www.openolat.org">
* OpenOLAT - Online Learning and Training</a><br>
* This file has been modified by the OpenOLAT community. Changes are licensed
* under the Apache 2.0 license as the original file.
*/

package org.olat.instantMessaging.ui;

import org.olat.core.CoreSpringFactory;
import org.olat.core.gui.UserRequest;
import org.olat.core.gui.components.form.flexible.FormItem;
import org.olat.core.gui.components.form.flexible.FormItemContainer;
import org.olat.core.gui.components.form.flexible.elements.MultipleSelectionElement;
import org.olat.core.gui.components.form.flexible.impl.FormBasicController;
import org.olat.core.gui.components.form.flexible.impl.FormEvent;
import org.olat.core.gui.components.form.flexible.impl.FormLayoutContainer;
import org.olat.core.gui.control.Controller;
import org.olat.core.gui.control.WindowControl;
import org.olat.instantMessaging.InstantMessagingModule;

/**
 *
 * @author gnaegi <www.goodsolutions.ch>
 * @author guido
 * Initial Date:  Aug 2, 2006
 * Description:
 * Instant messaging server administration task within olat
 * 
 */
public class InstantMessagingAdminController extends FormBasicController {

	private MultipleSelectionElement imEnabledEl;
	private MultipleSelectionElement imEnableGroupEl;
	private MultipleSelectionElement imEnableGroupAnonymEl;
	private MultipleSelectionElement imEnableGroupAnonymDefaultEl;
	private MultipleSelectionElement imEnableCourseEl;
	private MultipleSelectionElement imEnableCourseAnonymEl;
	private MultipleSelectionElement imEnableCourseAnonymDefaultEl;
	private MultipleSelectionElement imEnablePrivateEl;
	private MultipleSelectionElement imEnableGroupPeersEl;
	private MultipleSelectionElement imEnableOnlineStatusEl;
	
	private static String[] enabledKeys = new String[]{"on"};
	
	private final InstantMessagingModule imModule;

	public InstantMessagingAdminController(UserRequest ureq, WindowControl wControl) {
		super(ureq, wControl, "admin");
		
		imModule = CoreSpringFactory.getImpl(InstantMessagingModule.class);
		
		initForm(ureq);
	}
	
	@Override
	protected void initForm(FormItemContainer formLayout, Controller listener, UserRequest ureq) {
		//enable all
		FormLayoutContainer moduleFlc = FormLayoutContainer.createDefaultFormLayout("flc_module", getTranslator());
		formLayout.add(moduleFlc);
	
		String[] enabledValues = new String[]{ translate("enabled") };
		imEnabledEl = uifactory.addCheckboxesHorizontal("im.module.enabled", moduleFlc, enabledKeys, enabledValues, null);
		imEnabledEl.select(enabledKeys[0], imModule.isEnabled());
		imEnabledEl.addActionListener(listener, FormEvent.ONCHANGE);
		
		//options
		FormLayoutContainer chatOptionsFlc = FormLayoutContainer.createDefaultFormLayout("flc_chatOptions", getTranslator());
		formLayout.add(chatOptionsFlc);

		imEnableGroupEl = uifactory.addCheckboxesHorizontal("im.module.enabled.group", chatOptionsFlc, enabledKeys, enabledValues, null);
		imEnableGroupEl.select(enabledKeys[0], imModule.isGroupEnabled());
		imEnableGroupEl.addActionListener(listener, FormEvent.ONCHANGE);

		imEnableGroupAnonymEl = uifactory.addCheckboxesHorizontal("im.module.enabled.group.anonym", chatOptionsFlc, enabledKeys, enabledValues, null);
		imEnableGroupAnonymEl.select(enabledKeys[0], imModule.isGroupAnonymEnabled());
		imEnableGroupAnonymEl.addActionListener(listener, FormEvent.ONCHANGE);

		imEnableGroupAnonymDefaultEl = uifactory.addCheckboxesHorizontal("im.module.enabled.group.anonym.default", chatOptionsFlc, enabledKeys, enabledValues, null);
		imEnableGroupAnonymDefaultEl.select(enabledKeys[0], imModule.isGroupAnonymDefaultEnabled());
		imEnableGroupAnonymDefaultEl.addActionListener(listener, FormEvent.ONCHANGE);

		uifactory.addSpacerElement("spacer", chatOptionsFlc, true);
		
		imEnableCourseEl = uifactory.addCheckboxesHorizontal("im.module.enabled.course", chatOptionsFlc, enabledKeys, enabledValues, null);
		imEnableCourseEl.select(enabledKeys[0], imModule.isCourseEnabled());
		imEnableCourseEl.addActionListener(listener, FormEvent.ONCHANGE);

		imEnableCourseAnonymEl = uifactory.addCheckboxesHorizontal("im.module.enabled.course.anonym", chatOptionsFlc, enabledKeys, enabledValues, null);
		imEnableCourseAnonymEl.select(enabledKeys[0], imModule.isCourseAnonymEnabled());
		imEnableCourseAnonymEl.addActionListener(listener, FormEvent.ONCHANGE);

		imEnableCourseAnonymDefaultEl = uifactory.addCheckboxesHorizontal("im.module.enabled.course.anonym.default", chatOptionsFlc, enabledKeys, enabledValues, null);
		imEnableCourseAnonymDefaultEl.select(enabledKeys[0], imModule.isCourseAnonymDefaultEnabled());
		imEnableCourseAnonymDefaultEl.addActionListener(listener, FormEvent.ONCHANGE);

		FormLayoutContainer messageOptionsFlc = FormLayoutContainer.createDefaultFormLayout("flc_messageOptions", getTranslator());
		formLayout.add(messageOptionsFlc);
		
 		imEnablePrivateEl = uifactory.addCheckboxesHorizontal("im.module.enabled.private", messageOptionsFlc, enabledKeys, enabledValues, null);
		imEnablePrivateEl.select(enabledKeys[0], imModule.isPrivateEnabled());
		imEnablePrivateEl.addActionListener(listener, FormEvent.ONCHANGE);
		
		imEnableGroupPeersEl = uifactory.addCheckboxesHorizontal("im.module.enabled.grouppeers", messageOptionsFlc, enabledKeys, enabledValues, null);
		imEnableGroupPeersEl.select(enabledKeys[0], imModule.isGroupPeersEnabled());
		imEnableGroupPeersEl.addActionListener(listener, FormEvent.ONCHANGE);
		
		imEnableOnlineStatusEl = uifactory.addCheckboxesHorizontal("im.module.enabled.onlineStatus", messageOptionsFlc, enabledKeys, enabledValues, null);
		imEnableOnlineStatusEl.select(enabledKeys[0], imModule.isOnlineStatusEnabled());
		imEnableOnlineStatusEl.addActionListener(listener, FormEvent.ONCHANGE);
		
		// update GUI dependencies
		updateDependencies();
	}
	
	private void updateDependencies() {
		// disable all options when the module is turned off
		if (imEnabledEl.isSelected(0)) {
			imEnableGroupEl.setEnabled(true);
			imEnableGroupAnonymEl.setEnabled(true);
			if (imEnableGroupAnonymEl.isSelected(0)) { 
				imEnableGroupAnonymDefaultEl.setEnabled(true);
			} else {
				imEnableGroupAnonymDefaultEl.setEnabled(false);				
			}
			imEnableCourseEl.setEnabled(true);
			imEnableCourseAnonymEl.setEnabled(true);
			if (imEnableCourseAnonymEl.isSelected(0)) { 
				imEnableCourseAnonymDefaultEl.setEnabled(true);
			} else {
				imEnableCourseAnonymDefaultEl.setEnabled(false);
			}
			imEnablePrivateEl.setEnabled(true);
			imEnableGroupPeersEl.setEnabled(true);
			imEnableOnlineStatusEl.setEnabled(true);
		} else {
			// everything is disabled
			imEnableGroupEl.setEnabled(false);
			imEnableGroupAnonymEl.setEnabled(false);
			imEnableGroupAnonymDefaultEl.setEnabled(false);
			imEnableCourseEl.setEnabled(false);
			imEnableCourseAnonymEl.setEnabled(false);
			imEnableCourseAnonymDefaultEl.setEnabled(false);
			imEnablePrivateEl.setEnabled(false);
			imEnableGroupPeersEl.setEnabled(false);
			imEnableOnlineStatusEl.setEnabled(false);
		}
	}
	
	protected void doDispose() {
		// nothing to dispose
	}
	
	@Override
	protected void formInnerEvent(UserRequest ureq, FormItem source, FormEvent event) {
		if(source == imEnabledEl) {
			imModule.setEnabled(imEnabledEl.isSelected(0));
		} else if(source == imEnableGroupEl) {
			imModule.setGroupEnabled(imEnableGroupEl.isSelected(0));
		} else if(source == imEnableGroupAnonymEl) {
			imModule.setGroupAnonymEnabled(imEnableGroupAnonymEl.isSelected(0));
		} else if(source == imEnableGroupAnonymDefaultEl) {
			imModule.setGroupAnonymDefaultEnabled(imEnableGroupAnonymDefaultEl.isSelected(0));
		} else if(source == imEnableCourseEl) {
			imModule.setCourseEnabled(imEnableCourseEl.isSelected(0));
		} else if(source == imEnableCourseAnonymEl) {
			imModule.setCourseAnonymEnabled(imEnableCourseAnonymEl.isSelected(0));
		} else if(source == imEnableCourseAnonymDefaultEl) {
			imModule.setCourseAnonymDefaultEnabled(imEnableCourseAnonymDefaultEl.isSelected(0));
		} else if(source == imEnablePrivateEl) {
			imModule.setPrivateEnabled(imEnablePrivateEl.isSelected(0));
		} else if(source == imEnableGroupPeersEl) {
			imModule.setGroupPeersEnabled(imEnableGroupPeersEl.isSelected(0));
		} else if(source == imEnableOnlineStatusEl) {
			imModule.setOnlineStatusEnabled(imEnableOnlineStatusEl.isSelected(0));
		}
		// update GUI dependencies
		updateDependencies();
		
		super.formInnerEvent(ureq, source, event);
	}

	@Override
	protected void formOK(UserRequest ureq) {
		//
	}
}